﻿using System.Collections;
using UnityEngine;

namespace hry.labs.Player
{

    [RequireComponent(typeof(CharacterController))]
    public class Player : MonoBehaviour
    {

        public PlayerData playerData;
        public Combat.LaserGun laserGun;
        public UI.HealthBar healthBarPrefab;

        private CharacterController _playerCharacterController;
        private UI.HealthBar _healthBar;
        private Vector3 _moveVelocity = Vector3.zero;
        private int _currentHP = 100;


        void Start()
        {
            _playerCharacterController = GetComponent<CharacterController>();
            _healthBar = GameObject.Instantiate<UI.HealthBar>(healthBarPrefab, transform.position, Quaternion.identity);
            _currentHP = playerData.HP;

            // TODO 8 No spawning, so set hard time limit
        }

        IEnumerator EndGame()
        {
            yield return new WaitForSeconds(30.0f);
            Control.GameManager.Instance.GameEnded();
        }

        void Update()
        {
            UpdateVelocity();
            _playerCharacterController.Move(_moveVelocity * Time.deltaTime);

            UpdateRotation();

            // Use left mouse button to shoot lasers from blaster
            if (Input.GetMouseButtonDown(Utility.Constants.MOUSE_BUTTON_LEFT))
            {
                // TODO 3 Shoot to where the user clicked, ignore player and lasers
                Ray ray = UnityEngine.Camera.main.ScreenPointToRay(Vector3.zero);
                RaycastHit hit;
                // Hit anything but player
                if (Physics.Raycast(ray, out hit, 100.0f))
                {
                    laserGun.ShootLaser(hit.point, Mathf.Abs(Mathf.Sqrt(Vector3.Dot(_moveVelocity, new Vector3(1f,0f,1f)))));
                }
            }
        }

        private void LateUpdate()
        {
            _healthBar.transform.position = transform.position;
        }

        private void UpdateRotation()
        {
            if (Input.GetMouseButton(Utility.Constants.MOUSE_BUTTON_RIGHT))
            {
                Vector3 dir = Input.mousePosition - UnityEngine.Camera.main.WorldToScreenPoint(transform.position);
                transform.rotation = Quaternion.LookRotation(new Vector3(dir.x, 0.0f, dir.y));
            }
        }

        private void UpdateVelocity()
        {
            // TODO 2 Compute velocity
            // Check if on character is on the ground and if so, add velocities
            // Get vertical and horizontal axis speed and jump to the _moveVelocity
            // If player is grounded and jumps, add y-axis speed

            // Apply gravity acceleration
        }

        private void OnTriggerEnter(Collider other)
        {
            // TODO 8 Die on destroy!
            if (other.gameObject.CompareTag(Utility.Constants.TAG_LASER))
            {
                _currentHP -= 10;
                _healthBar.UpdateHealth(Mathf.Max(0.0f, (float)_currentHP / playerData.HP));
                Destroy(other.gameObject);

                if (_currentHP <= 0)
                {
                }
            }
        }
    }

}
